<?php

namespace common\logic\CreditHistory;

use common\logic\BaseService;
use common\logic\BuilderInterface;
use common\services\UserProducer\UserProducerBuilder;

class CreditHistoryBuilder extends BaseService implements BuilderInterface
{
    protected CreditHistorySolver $creditHistorySolver;
    protected UserProducerBuilder $userProducerBuilder;

    public function __construct()
    {
        $this->creditHistorySolver = $this->loadService(CreditHistorySolver::class);
        $this->userProducerBuilder = $this->loadService(UserProducerBuilder::class);
    }

    public function save(CreditHistoryModel $creditHistory): bool
    {
        if ($creditHistory->getAmount() > -0.01 && $creditHistory->getAmount() < 0.01) {
            return false;
        }

        // Initialisation du commentaire avant sauvegarde
        $creditHistory->setComment($creditHistory->getComment() . $this->creditHistorySolver->getStrComment($creditHistory));

        $creditHistory->save();

        // Mise à jour du crédit au niveau de UserProducer
        $this->userProducerBuilder->updateCredit($creditHistory);

        return true;
    }
}