<?php

namespace common\logic\CreditHistory;

use common\helpers\MeanPayment;
use common\logic\SolverInterface;
use common\models\Order;
use yii\helpers\Html;

class CreditHistorySolver implements SolverInterface
{

    public function isTypeDebit(CreditHistoryModel $creditHistory): bool
    {
        return in_array($creditHistory->type, [
            CreditHistoryModel::TYPE_DEBIT,
            CreditHistoryModel::TYPE_PAYMENT,
        ]);
    }

    public function isTypeCredit(CreditHistoryModel $creditHistory): bool
    {
        return in_array($creditHistory->type, [
            CreditHistoryModel::TYPE_CREDIT,
            CreditHistoryModel::TYPE_INITIAL_CREDIT,
            CreditHistoryModel::TYPE_REFUND
        ]);
    }

    public function getAmount(CreditHistoryModel $creditHistory, bool $format = false): string
    {
        if ($format) {
            return number_format($creditHistory->getAmount(), 2) . '&nbsp;€';
        } else {
            return $creditHistory->getAmount();
        }
    }

    /**
     * Retourne le libellé du CreditHistory informant de son type et
     * éventuellement de la date de sa commande associée.
     *
     */
    public function getStrWording(CreditHistoryModel $creditHistory): string
    {
        $str = '';
        $type = $creditHistory->getType();

        if (CreditHistoryModel::TYPE_INITIAL_CREDIT == $type) {
            $str = 'Crédit initial';
        } elseif (CreditHistoryModel::TYPE_CREDIT == $type) {
            $str = 'Crédit';
        } elseif (CreditHistoryModel::TYPE_PAYMENT == $type) {
            $str = 'Paiement';
        } elseif (CreditHistoryModel::TYPE_REFUND == $type) {
            $str = 'Remboursement';
        } elseif (CreditHistoryModel::TYPE_DEBIT == $type) {
            $str = 'Débit';
        }

        if (CreditHistoryModel::TYPE_PAYMENT == $type || CreditHistoryModel::TYPE_REFUND == $type) {
            $order = $creditHistory->getOrderObject();
            if ($order && $order->getDistributionOject()) {
                $str .= '<br />Commande : ' . date('d/m/Y', strtotime($order->getDistributionOject()->getDate()));
            } else {
                $str .= '<br />Commande supprimée';
            }
        }

        return $str;
    }

    /**
     * Retourne les informations à ajouter au commentaire du CreditHistorique
     * (libellé, montant, client, action) au format HTML.
     *
     */
    public function getStrComment(CreditHistoryModel $creditHistory): string
    {
        $str = '';
        if (strlen($creditHistory->getComment())) {
            $str .= '<br />';
        }

        $str .= $this->getStrWording($creditHistory);

        $order = $creditHistory->getOrderObject();
        if ($order) {
            $str .= '<br />Montant de la commande : ' . $order->getAmountWithTax(Order::AMOUNT_TOTAL, true);
        }

        $user = $creditHistory->getUserObject();
        if ($user) {
            $str .= '<br />Client : ' . Html::encode($user->getName() . ' ' . $user->getLastname());
        }

        $userAction = $creditHistory->getUserActionObject();
        if ($userAction) {
            $str .= '<br />Action : ' . Html::encode($userAction->getName() . ' ' . $userAction->getLastname());
        }

        return $str;
    }

    public function getDate(CreditHistoryModel $creditHistory, bool $format = false): string
    {
        $date = $creditHistory->getDate();

        if ($format) {
            return date('d/m/Y à H:i:s', strtotime($date));
        } else {
            return $date;
        }
    }

    public function getStrMeanPayment(CreditHistoryModel $creditHistory): string
    {
        return MeanPayment::getStrBy($creditHistory->getMeanPayment());
    }

    // strUserAction
    public function getStrUserAction(CreditHistoryModel $creditHistory): string
    {
        $userAction = $creditHistory->getUserActionObject();

        if ($userAction) {
            return $userAction->getName() . ' ' . $userAction->getlastname();
        } else {
            return 'Système';
        }
    }
}